/*
 * Copyright (c) 2011 Petr Koupy
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup kgraph
 * @{
 */
/**
 * @file
 */

#include <errno.h>
#include <stdio.h>
#include <loc.h>
#include <async.h>
#include <task.h>
#include <libgraph.h>
#include <ops/graph_dev.h>
#include "port/ega.h"
#include "port/kchar.h"
#include "port/kfb.h"
#include "port/niagara.h"
#include "port/ski.h"
#include "kgraph.h"

static int kgraph_dev_add(ddf_dev_t *dev)
{
	ega_init(dev);
	kchar_init(dev);
	kfb_init(dev);
	niagara_init(dev);
	ski_init(dev);

	printf("%s: Accepting connections\n", NAME);
	return EOK;
}

static driver_ops_t kgraph_driver_ops = {
	.dev_add = kgraph_dev_add,
};

static driver_t kgraph_driver = {
	.name = NAME,
	.driver_ops = &kgraph_driver_ops
};

static graph_dev_ops_t graph_vsl_dev_ops = {
	.connect = (connect_func) &graph_visualizer_connection
};

ddf_dev_ops_t graph_vsl_device_ops = {
	.interfaces[GRAPH_DEV_IFACE] = &graph_vsl_dev_ops
};

static graph_dev_ops_t graph_rnd_dev_ops = {
	.connect = (connect_func) &graph_renderer_connection
};

ddf_dev_ops_t graph_rnd_device_ops = {
	.interfaces[GRAPH_DEV_IFACE] = &graph_rnd_dev_ops
};

int main(int argc, char *argv[])
{
	printf("%s: HelenOS Kernel Graphics Driver\n", NAME);
	return ddf_driver_main(&kgraph_driver);
}

/** @}
 */
