/*
 * Copyright (c) 2011 Petr Koupy
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup libc
 * @{
 */
/**
 * @file
 */

#ifndef LIBC_IO_CHARFIELDMAP_H_
#define LIBC_IO_CHARFIELDMAP_H_

#include <sys/types.h>
#include <io/charfield.h>

typedef struct {
	sysarg_t cols;
	sysarg_t rows;
	charfield_t *data;
} charfieldmap_t;

static inline charfield_t *charfieldmap_charfield_at(
    charfieldmap_t *charfieldmap,
    sysarg_t col,
    sysarg_t row)
{
	size_t offset = row * charfieldmap->cols + col;
	charfield_t *field = charfieldmap->data + offset;
	return field;
}

static inline void charfieldmap_put_charfield(
    charfieldmap_t * charfieldmap,
    sysarg_t col,
    sysarg_t row,
    charfield_t charfield)
{
	charfield_t *target = charfieldmap_charfield_at(charfieldmap, col, row);
	*target = charfield;
}

static inline charfield_t charfieldmap_get_charfield(
    charfieldmap_t *charfieldmap,
    sysarg_t col,
    sysarg_t row)
{
	charfield_t *source = charfieldmap_charfield_at(charfieldmap, col, row);
	return *source;
}

#endif

/** @}
 */
