/*
 * Copyright (c) 2012 Petr Koupy
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup draw
 * @{
 */
/**
 * @file
 */

#ifndef DRAW_FONT_H_
#define DRAW_FONT_H_

#include <sys/types.h>

#include "surface.h"
#include "source.h"

struct drawctx;
typedef struct drawctx drawctx_t;

typedef enum {
	FONT_DECODER_EMBEDDED
} font_decoder_type_t;

typedef struct {
	void (*init)(char *, uint16_t *, void **);
	uint16_t (*resolve)(const wchar_t, void *);
	surface_t *(*render)(uint16_t, uint16_t);
	void (*release)(void *);
} font_decoder_t;

typedef struct font {
	uint16_t points;
	uint16_t glyph_count;
	surface_t **glyphs;
	font_decoder_t *decoder;
	void *decoder_data;
} font_t;

extern void font_init(font_t *, font_decoder_type_t, char *, uint16_t);
extern void font_release(font_t *);

extern void font_get_box(font_t *, char *, sysarg_t *, sysarg_t *);
extern void font_draw_text(font_t *, drawctx_t *, source_t *, char *, sysarg_t, sysarg_t);

#endif

/** @}
 */
