/*
 * Copyright (c) 2012 Petr Koupy
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup gui
 * @{
 */
/**
 * @file
 */

#ifndef GUI_TERMINAL_H_
#define GUI_TERMINAL_H_

#include <bool.h>
#include <sys/types.h>
#include <fibril_synch.h>
#include <adt/list.h>
#include <io/mode.h>
#include <task.h>

#include <surface.h>

#include <libgraph.h>

#include "widget.h"

typedef struct {
	link_t link;
	service_id_t dsid; /**< Service ID of the input emulator. */
	async_sess_t *sess; /**< Input session for the console. */
} terminal_input_relay_t;

typedef struct terminal {
	widget_t widget;
	visualizer_t *visualizer; /**< Output emulator based on libgraph. */
	vslmode_list_element_t mode; /**< Dynamic mode mirrors widget properties. */
	bool buffered; /**< Terminal can be either buffered or unbuffered. */
	surface_t *buffer; /**< Optional terminal buffer. */
	fibril_mutex_t buffer_mtx; /**< Mutex protecting the buffer. */
	terminal_input_relay_t relay; /**< Simple input emulator. */
	sysarg_t hpos_last; /**< Console expects relative mouse changes. */
	sysarg_t vpos_last; /**< Console expects relative mouse changes. */
	task_id_t console_task; /**< Remote console task. */
} terminal_t;

/**
 * Terminal can be either buffered or unbuffered. Unbuffered version obviously
 * provides more performance but it might not work well in complex widget
 * hiearachy (it is mainly intended for whole-window terminal). Also the number
 * of console terminal tabs can be spcified. Optionally, last terminal tab
 * might be reserved for klog.
 **/
extern bool init_terminal(
    terminal_t *, widget_t *, bool, sysarg_t, sysarg_t, sysarg_t, bool);
extern terminal_t *create_terminal(
    widget_t *, bool, sysarg_t, sysarg_t, sysarg_t, bool);
extern void deinit_terminal(terminal_t *);

#endif

/** @}
 */

