/*
 * Copyright (c) 2011 Petr Koupy
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup libposix
 * @{
 */
/** @file Support for waiting.
 */

#ifndef POSIX_SYS_WAIT_H_
#define POSIX_SYS_WAIT_H_

#include "types.h"

#undef WIFEXITED
#undef WEXITSTATUS
#undef WIFSIGNALED
#undef WTERMSIG
#define WIFEXITED(status) __posix_wifexited(status)
#define WEXITSTATUS(status) __posix_wexitstatus(status)
#define WIFSIGNALED(status) __posix_wifsignaled(status)
#define WTERMSIG(status) __posix_wtermsig(status)

extern int __posix_wifexited(int status);
extern int __posix_wexitstatus(int status);
extern int __posix_wifsignaled(int status);
extern int __posix_wtermsig(int status);

extern posix_pid_t posix_wait(int *stat_ptr);
extern posix_pid_t posix_waitpid(posix_pid_t pid, int *stat_ptr, int options);

#ifndef LIBPOSIX_INTERNAL
	#define wait posix_wait
	#define waitpid posix_waitpid
#endif

#endif /* POSIX_SYS_WAIT_H_ */

/** @}
 */
