/*
 * Copyright (c) 2006 Josef Cejka
 * Copyright (c) 2011 Petr Koupy
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup console
 * @{
 */
/** @file
 */

#ifndef CONSOLE_SCROLLGRID_H_
#define CONSOLE_SCROLLGRID_H_

#include <sys/types.h>
#include <bool.h>
#include <io/charfieldmap.h>

struct scrollgrid;
typedef struct scrollgrid scrollgrid_t;

typedef enum {
	SCROLLGRID_FLAG_NONE = 0,
	SCROLLGRID_FLAG_SHARED = 1
} scrollgrid_flag_t;

extern scrollgrid_t *scrollgrid_create(sysarg_t, sysarg_t, scrollgrid_flag_t);
extern void scrollgrid_destroy(scrollgrid_t *);
extern void scrollgrid_copy_content(scrollgrid_t *, scrollgrid_t *);

extern bool scrollgrid_is_shared(scrollgrid_t *);
extern charfield_t *scrollgrid_direct_access(scrollgrid_t *);
extern void scrollgrid_get_dimensions(scrollgrid_t *, sysarg_t *, sysarg_t *);
extern void scrollgrid_get_damaged_region(scrollgrid_t *, sysarg_t *, sysarg_t *,
    sysarg_t *, sysarg_t *);
extern void scrollgrid_reset_damaged_region(scrollgrid_t *);
extern sysarg_t scrollgrid_get_top_row(scrollgrid_t *);

extern void scrollgrid_put_field(scrollgrid_t *, sysarg_t, sysarg_t, charfield_t);
extern charfield_t scrollgrid_get_field(scrollgrid_t *, sysarg_t, sysarg_t);

extern void scrollgrid_clear(scrollgrid_t *);
extern bool scrollgrid_putchar(scrollgrid_t *, wchar_t);
extern bool scrollgrid_newline(scrollgrid_t *);
extern bool scrollgrid_tabstop(scrollgrid_t *, sysarg_t);
extern bool scrollgrid_backspace(scrollgrid_t *);

extern void scrollgrid_set_cursor(scrollgrid_t *, sysarg_t, sysarg_t);
extern void scrollgrid_get_cursor(scrollgrid_t *, sysarg_t *, sysarg_t *);
extern void scrollgrid_set_cursor_visibility(scrollgrid_t *, bool);
extern bool scrollgrid_get_cursor_visibility(scrollgrid_t *);

extern void scrollgrid_set_pointer(scrollgrid_t *, sysarg_t, sysarg_t);
extern void scrollgrid_get_pointer(scrollgrid_t *, sysarg_t *, sysarg_t *);
extern void scrollgrid_set_pointer_visibility(scrollgrid_t *, bool);
extern bool scrollgrid_get_pointer_visibility(scrollgrid_t *);

extern void scrollgrid_set_cursor_attrs(scrollgrid_t *, char_attrs_t);
extern void scrollgrid_set_pointer_attrs(scrollgrid_t *, char_attrs_t);
extern void scrollgrid_set_default_attrs(scrollgrid_t *, char_attrs_t);

#endif

/** @}
 */
